local addonName, ns = ...

local itemsSlotTable = {
	15,	--INVSLOT_BACK
	9,	--INVSLOT_WRIST
	10,	--INVSLOT_HAND
	6,	--INVSLOT_WAIST
	7,	--INVSLOT_LEGS
	8,	--INVSLOT_FEET
	11,	--INVSLOT_FINGER1
	12,	--INVSLOT_FINGER2
	13,	--INVSLOT_TRINKET1
	14,	--INVSLOT_TRINKET2
	16,	--INVSLOT_MAINHAND
	17,	--INVSLOT_OFFHAND
}

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(self, event, ...)
	return self[event] and self[event](self, event, ...)
end)
f:RegisterEvent("ADDON_LOADED")

local function Print(text, ...)
	if text then
		if text:match("%%[dfqs%d%.]") then
			DEFAULT_CHAT_FRAME:AddMessage("|cffffcc00".. addonName ..":|r " .. format(text, ...))
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffffcc00".. addonName ..":|r " .. strjoin(" ", text, tostringall(...)))
		end
	end
end

function f:ADDON_LOADED(event, addon)
	if addon == addonName then
		f:RegisterEvent("CHAT_MSG_WHISPER")
		Print('Loaded')
	end
end

function f:CHAT_MSG_WHISPER(msg, player, ...)
	if string.find(msg, '+') or string.find(msg, 'invite') then
		InviteUnit(player)
	end
end

function CheckTanks()
	local n = GetNumGroupMembers() or 0

	local inRaid = IsInRaid()
	local inGroup = IsInGroup()

	local withoutWardenEssence = {}

	if n < 1 then
		if not CheckTankEssences('player') then withoutWardenEssence[#withoutWardenEssence + 1] = UnitName('player') end
	else
		for i=1,n do
			local name,subgroup,_,unit,online
			
			online = true
			
			if not inRaid or i <= 5 then
				unit = i == 1 and 'player' or 'party'..(i-1)
				name = UnitName(unit)
				subgroup = 1
			else
				name,_,subgroup,_,_,_,_,online = GetRaidRosterInfo(i)
				unit = "raid"..i
			end

			if online and UnitGroupRolesAssigned(unit) == "TANK" and CheckInteractDistance(unit, 1) then
				--if CheckInteractDistance(unit, 1) or i == 1 then
					if not CheckTankEssences(unit) then withoutWardenEssence[#withoutWardenEssence + 1] = UnitName(unit) end
					--if i > 1 then
						--NotifyInspect(unit)
					--else
						--CheckTankCorruption(name)
					--end
				--end
			end
		end
	end
	
	local msg = "WITHOUT WARDEN ESSENCE RANK 3: "
	
	if #withoutWardenEssence > 0 then
		for i=1,#withoutWardenEssence do
			if i > 1 then
				msg = msg..", "..withoutWardenEssence[i]
			else
				msg = msg..withoutWardenEssence[i]
			end
		end
		
		if inRaid then
			SendChatMessage(msg, string.upper("raid"))
		elseif inGroup then
			SendChatMessage(msg, string.upper("party"))
		else
			Print(msg)
		end
	end
end

function f:INSPECT_READY(guid)
	local _,_,_,_,_,name,realm = GetPlayerInfoByGUID(guid)
	if realm and realm ~= "" then name = name.."-"..realm end
	CheckTankCorruption(name)
end

function CheckTankEssences(unit)
	local name = UnitName(unit)
	local wardenRank3Found = "NO"
	local animaEssenceFound = "NO"
	local neededEssencesCount = 0

	local wardenFound = false

	for i=1,40 do
		local buffName, _, _, _, _, _, unitCaster, _, _, spellId = UnitAura(name, i, "HELPFUL")
		
		if not spellId then
			break
		else
			if spellId == 312107 and unitCaster == unit then
				wardenRank3Found = "YES"
				wardenFound = true
				neededEssencesCount = neededEssencesCount + 1
			elseif spellId == 294966 and unitCaster == unit then
				animaEssenceFound = "YES"
				neededEssencesCount = neededEssencesCount + 1
			end

			if neededEssencesCount >= 2 then break end
		end
	end
	
	local msg = name..' has Warden Essence Rank 3: '..wardenRank3Found..', has Anima Essence: '..animaEssenceFound
	
	if IsInRaid(unit) then
		SendChatMessage(msg, string.upper("raid"))
	elseif IsInGroup(unit) then
		SendChatMessage(msg, string.upper("party"))
	else
		Print(msg)
	end
	
	return wardenFound
end

function CheckTankCorruption(name)
	local corruptions = GetCharacterCorruptions(name)
	local summaryTD = 0
	local summaryEchoingVoid = 0

	if #corruptions > 0 then
		for i=1, #corruptions do
			local corruptionBonus = corruptions[i]

			if 	   string.find(corruptionBonus, "6539") then
				summaryTD = summaryTD + 18
			elseif string.find(corruptionBonus, "6538") then
				summaryTD = summaryTD + 12
			elseif string.find(corruptionBonus, "6537") then
				summaryTD = summaryTD + 6
			elseif string.find(corruptionBonus, "6549") then
				summaryEchoingVoid = summaryEchoingVoid + 0.4
			elseif string.find(corruptionBonus, "6550") then
				summaryEchoingVoid = summaryEchoingVoid + 0.6
			elseif string.find(corruptionBonus, "6551") then
				summaryEchoingVoid = summaryEchoingVoid + 1
			end
		end
	end
	
	local msg = name..' has '..summaryTD..'% devastation, '..summaryEchoingVoid..'% echoing void'
	
	if IsInRaid(unit) then
		SendChatMessage(msg, string.upper("raid"))
	elseif IsInGroup(unit) then
		SendChatMessage(msg, string.upper("party"))
	else
		Print(msg)
	end
end

function GetItemSplit(itemLink)
	local itemString = string.match(itemLink, "item:([%-?%d:]+)")
	local itemSplit = {}

	-- Split data into a table
	for _, v in ipairs({strsplit(":", itemString)}) do
		if v == "" then
			itemSplit[#itemSplit + 1] = 0
		else
			itemSplit[#itemSplit + 1] = tonumber(v)
		end
	end

	return itemSplit
end

function GetCharacterCorruptions(name)
	local corruptions = {}

    for i=1,#itemsSlotTable do
        local itemLink = GetInventoryItemLink(name, itemsSlotTable[i])

        if itemLink then
            local itemSplit = GetItemSplit(itemLink)

            for index=1, itemSplit[13] do
                corruptions[#corruptions + 1] = itemSplit[13 + index]
            end
        end
    end

	ClearInspectPlayer()

    return corruptions
end

-- Slash
SLASH_TANKSEVENTSHELPER1 = "/tanks"

local SlashHandlers = {
	["check"] = function()
		Print("Checking tanks...")
		CheckTanks()
	end,
	["help"] = function()
		Print(" /tanks check     <<< checks Warden and Anima essences of the each tank in your group or raid")
	end,
}

SlashCmdList["TANKSEVENTSHELPER"] = function(text)
	local command, params = strsplit(" ", text, 2)

	if SlashHandlers[command] then
		SlashHandlers[command](params)
	end
end